(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



(* ::Subsection:: *)
(*Begin Package*)


BeginPackage["AdiabaticTools`",
{"Global`",
"SpinDynamica`",
"QM`Hilbert`",
"QM`SpinOperators`",
"QM`Liouville`",
"SD`CorrelatedEigensystems`"}
];


adiabaticprofile::usage = "The function \!\(\*
StyleBox[\"adiabaticprofile\",\nFontSlant->\"Italic\"]\) generates and adiabatic profile in agreement with the constant adiabaticity approach described in: https://doi.org/10.1063/1.5079436.

The syntax for \!\(\*
StyleBox[\"adiabaticprofile\",\nFontSlant->\"Italic\"]\) is given by: adiabaticprofile[{\!\(\*SubscriptBox[\(H\), \(0\)]\), \!\(\*SubscriptBox[\(Q\), \(ad\)]\)}, {x, \!\(\*SubscriptBox[\(x\), \(min\)]\), \!\(\*SubscriptBox[\(x\), \(max\)]\), \[Delta]x}, {\!\(\*SubscriptBox[\(\[Xi]\), \(target\)]\), adiabatic_states}].

*) \!\(\*SubscriptBox[\(H\), \(0\)]\) represents the time-independent part of the Hamiltonian.
*) \!\(\*SubscriptBox[\(Q\), \(ad\)]\) represents the adiabatic generator.

*) x represents the adiabatic coordinate.
*) \!\(\*SubscriptBox[\(x\), \(min\)]\), \!\(\*SubscriptBox[\(x\), \(max\)]\) and \[Delta]x represent the initial value, the final value and the increment of the adiabatic coordinate, respectively.

*) \!\(\*SubscriptBox[\(\[Xi]\), \(target\)]\) represents the target adiabaticity parameter such that \[Xi](t) = \!\(\*SubscriptBox[\(\[Xi]\), \(target\)]\) \[ForAll] t \[GreaterEqual] 0. The target adiabaticity \!\(\*SubscriptBox[\(\[Xi]\), \(target\)]\) usually stretches or contracts the time-axis.
*) adiabatic_states represents the set of adiabatic states that should be included in the calculation of the generalised adiabaticity parameter. Relevant adiabatic states may be determined by considering the eigenvalues returned by CorrelatedEigensystems, for example.

For a typical SLIC experiment with Hamiltonian:  

H(\!\(\*SubscriptBox[\(\[Nu]\), \(x\)]\)) = J opI[1].opI[2] + \[CapitalDelta]/2 (opI[1,z]-opI[2,z]) + \!\(\*SubscriptBox[\(\[Nu]\), \(x\)]\) opI[x],

the adiabatic profile with constant adiabaticity parameter would be calculated as follows:

adiabaticprofile[{50 opI[1].opI[2] + 10/2 (opI[1,z]-opI[2,z]), opI[x]}, {\!\(\*SubscriptBox[\(\[Nu]\), \(x\)]\), 0, 100, 1}, {.5, {1,4}}]

The output generated by \!\(\*
StyleBox[\"adiabaticprofile\",\nFontSlant->\"Italic\"]\) takes the form {{evsystem,evecsystem},{\!\(\*SubscriptBox[\(t\), \(max\)]\), f[t]}}. 

*) evsystem is a system of correlated eigenvalues as returned by CorrelatedEigensystems.
*) evecsystem is a system of correlated eigenvectors as returned by CorrelatedEigensystems.

*) \!\(\*SubscriptBox[\(t\), \(max\)]\) is the time it takes the constant adiabaticity sweep to reach \!\(\*SubscriptBox[\(x\), \(max\)]\).
*) f[t] is the corresponding adiabatic profile with constant adiabaticity \!\(\*SubscriptBox[\(\[Xi]\), \(target\)]\).";


Begin["`Private`"];


(* ::Subsection:: *)
(*Input checks*)


adiabaticprofile::HnotQ= "The Hamiltonian is not an operator";
adiabaticprofile::QnotQ= "The adiabatic generator is not an operator";
adiabaticprofile::xdef= "The adiabatic coordinate is not undefined";
adiabaticprofile::xmindef= "The initial value for the adiabatic coordinate is not a number";
adiabaticprofile::xmaxdef=  "The final value for the adiabatic coordinate is not a number";
adiabaticprofile::dxdef=  "The increment for the adiabatic coordinate is not a number";
adiabaticprofile::\[Xi]def=  "The target adiabaticity parameter is not a number";
adiabaticprofile::statesdef=  "The set of adiabatic states is not a list of integers";


(* ::Subsection:: *)
(*main function*)


adiabaticprofile[{H0_,Qtd_},{x_,xmin_,xmax_,dx_},{\[Xi]0_,states_}]:=Module[{Qtdmat,Hmat,\[CapitalSigma]system,\[CapitalLambda]system,evraw,\[Omega]ij,Qtdin\[CapitalLambda],Qij,indexset,\[Xi],qtd,qprofile,tmax},
If[OperatorQ[H0], True, Message[adiabaticprofile::HnotQ];Abort[]];
If[OperatorQ[Qtd], True, Message[adiabaticprofile::QnotQ];Abort[]];
If[Length[UnassignedSymbols[x]]==1, True, Message[adiabaticprofile::xdef];Abort[]];
If[NumberQ[xmin], True, Message[adiabaticprofile::xmindef];Abort[]];
If[NumberQ[xmax], True, Message[adiabaticprofile::xmaxdef];Abort[]];
If[NumberQ[dx], True, Message[adiabaticprofile::dxdef];Abort[]];
If[NumberQ[\[Xi]0], True, Message[adiabaticprofile::\[Xi]def];Abort[]];
If[Quiet@MatchQ[Union[IntegerQ/@states],{True}], True, Message[adiabaticprofile::statesdef];Abort[]];
Hmat=MatrixRepresentation[H0+x Qtd];
{\[CapitalSigma]system,\[CapitalLambda]system}=CorrelatedEigensystems[Hmat,{x,xmin,5*dx+xmax,dx}];
evraw=Transpose[\[CapitalSigma]system][[All,All,2]];
\[Omega]ij[{i_,j_},k_]:=evraw[[k,i]]-evraw[[k,j]];
Qtdmat=MatrixRepresentation[Qtd];
Qtdin\[CapitalLambda]=#.Qtdmat.ConjugateTranspose[#]&/@\[CapitalLambda]system[[All,2]];
Qij[{i_,j_},k_]:=Qtdin\[CapitalLambda][[k]][[i,j]];
indexset=Flatten[Table[{i,j},{i,states},{j,states}],1]/.{{i_,i_}:> Nothing};
\[Xi][xint_]:=Evaluate[
Interpolation[
Transpose[
{\[CapitalLambda]system[[All,1]],Sqrt[Sum[If[MatchQ[Chop[\[Omega]ij[ij,#]],0],0,(Qij[ij,#]^2/\[Omega]ij[ij,#]^4)],{ij,indexset}]]&/@Range[1,Length[\[CapitalLambda]system]]}
]
][xint]
];
qprofile=qtd/.First@Quiet@NDSolve[{qtd'[t]==\[Xi]0*1/\[Xi][qtd[t]],qtd[0]==xmin,WhenEvent[qtd[t]==xmax,{tmax=t,"StopIntegration"}]},qtd,{t,0,\[Infinity]}];
{{\[CapitalSigma]system,\[CapitalLambda]system},{tmax,qprofile}}
];


(* ::Subsection:: *)
(*End package*)


Print["SpinDynamica constant adiabaticity addon 1.0"];


End[];
EndPackage[];
